<?php

class LEADIFLYADVANCED_SettingsController {
    public function __construct() {
        add_action('admin_menu', array($this, 'add_settings_menu'));
        add_action('admin_init', array($this, 'save'));

        add_action('admin_init', array($this, 'save'));

        add_action('added_post_meta', array($this, 'monitor_for_new_subscriber'), 10, 4 );
        add_action('updated_post_meta', array($this, 'monitor_for_new_subscriber'), 10, 4 );
    }

    function monitor_for_new_subscriber( $meta_id, $post_id, $meta_key, $meta_value ) {
        //look for leadifly facebook access token...
        if('leadifly_subscriber_access_token' == $meta_key ){
            $email = get_post_meta($post_id, 'leadifly_subscriber_email', true);
            $first_name = get_post_meta($post_id, 'leadifly_subscriber_first_name', true);
            $last_name = get_post_meta($post_id, 'leadifly_subscriber_last_name', true);

            $settings = LEADIFLYADVANCED_ChromeEngageNotifier::instance()->settings;

            if(!empty($settings->formcode)){
                $this->submit_form($settings->formcode, $email, $first_name, $last_name);
            }

            if($settings->send_welcome){
                $this->send_email($settings->welcome_from, $settings->welcome_from_name, $email, $settings->welcome_subject, $settings->welcome_body);
            }
        }
    }

    function submit_form($formcode, $email, $first_name, $last_name){
        $full_name = trim(implode(' ', array($first_name, $last_name)));

        try {
            $data = array();
            $url = false;
            $method = 'GET';

            $document = new DOMDocument();
            $temp = libxml_use_internal_errors(true);
            $document->loadHTML($formcode);
            libxml_use_internal_errors($temp);

            $inputs = $document->getElementsByTagName('input');

            $data = array();

            foreach ($inputs as $input) {
                $name = $input->getAttribute('name');
                $value = $input->getAttribute('value');

                if(!empty($name)){
                    $data[$name] = $value;
                }
            }

            $forms = $document->getElementsByTagName('form');

            foreach ($forms as $form) {
                $action = $form->getAttribute('action');
                $method_temp = $form->getAttribute('method');

                if(!empty($action)){
                    $url = $action;
                }

                if(!empty($method_temp)){
                    $method = strtoupper($method_temp);
                }
            }

            foreach(array('email', 'mail') as $name){
                if(array_key_exists($name, $data)){
                    $data[$name] = $email;
                }
            }

            foreach(array('first', 'firstname', 'first_name', 'fname') as $name){
                if(array_key_exists($name, $data)){
                    $data[$name] = $first_name;
                }
            }

            foreach(array('last', 'lastname', 'last_name', 'lname') as $name){
                if(array_key_exists($name, $data)){
                    $data[$name] = $last_name;
                }
            }

            foreach(array('name', 'fullname', 'full_name') as $name){
                if(array_key_exists($name, $data)){
                    $data[$name] = $full_name;
                }
            }

            $response = wp_remote_post(
                $url,
                array(
                    'method' => 'POST',
                    'timeout' => 45,
                    'body' => $data,
                )
            );
        }catch (Exception $e){}
    }

    function send_email($from, $name, $email, $subject, $body){
        $headers = array(
            "From: $name <$from>",
            'Content-Type: text/html; charset=UTF-8'
        );

        wp_mail( $email, $subject, $body, $headers );
    }

    function add_settings_menu(){
        add_options_page(
            LEADIFLYADVANCED_TITLE,
            LEADIFLYADVANCED_TITLE,
            'manage_options',
            LEADIFLYADVANCED_SLUG,
            array($this, 'render')
        );
    }

    public function save() {

        if (!current_user_can('manage_options')) {
            return;
        }

        if (isset($_POST['action']) && $_POST['action'] == 'update' && isset($_GET['page']) && $_GET['page'] == LEADIFLYADVANCED_SLUG) {
            $nonce = $_POST['_wpnonce'];
            if (!wp_verify_nonce($nonce, 'update')) {
                wp_die('Nice Try...');
            }

            $post = stripslashes_deep($_POST);

            $settings = LEADIFLYADVANCED_ChromeEngageNotifier::instance()->settings;

            $settings->formcode = $post['formcode'];
            $settings->welcome_from = $post['welcome_from'];
            $settings->welcome_from_name = $post['welcome_from_name'];
            $settings->welcome_subject = $post['welcome_subject'];
            $settings->welcome_body = $post['welcome_body'];
            $settings->send_welcome = array_key_exists('send_welcome', $post);
        }
    }

    public function render() {
        $settings = LEADIFLYADVANCED_ChromeEngageNotifier::instance()->settings;
        require LEADIFLYADVANCED_VIEW_DIR . '/settings.php';
    }
}